# This scripts prepares and saves the `citeseq` data.

args <- R.utils::commandArgs(
  trailingOnly = TRUE,
  asValues = TRUE
)

suppressMessages({
  library(TENxPBMCData)
  library(scater)
  library(Seurat)
  library(scran)
})

set.seed(3112022)

# Extensive processing of this data occurs in the download_data file, perhaps
# it should be moved here.
citeseq <- readRDS(args$raw_data)

rowData(citeseq) <- DataFrame(value = rownames(citeseq))

# Filter to the top 2000 genes.
dec_citeseq <- modelGeneVarByPoisson(citeseq)
top_citeseq <- getTopHVGs(dec_citeseq, n = 2000)
citeseq <- citeseq[top_citeseq, ]

# All cluster labels (not annotated).
colLabels(citeseq) <- factor(citeseq$RNA_snn_res.0.2)

saveRDS(citeseq, args$data)
